@@ -3,7 +3,9 @@ import json
 import logging
 from typing import Dict, Optional
 
+import websockets.client
 import websockets.exceptions
+import websockets.typing
 
 from classiq_interface.analyzer import analysis_params, result as analysis_result
 from classiq_interface.chemistry import ground_state_problem, operator
@@ -19,6 +21,8 @@ from classiq_interface.server import routes
 from classiq import client
 from classiq.exceptions import ClassiqExpiredTokenError, ClassiqValueError
 
+_FAIL_FAST_INDICATOR = "{"
+
 
 def _retry_websocket_on_exception(func):
     # TODO fix this workaround
@@ -37,6 +41,27 @@ def _retry_websocket_on_exception(func):
     return wrapped
 
 
+def _decode_message(msg: websockets.typing.Data) -> str:
+    return msg.decode() if isinstance(msg, bytes) else msg
+
+
+async def _get_return_value_from_websocket(
+    websocket: websockets.client.WebSocketClientProtocol,
+) -> str:
+    is_first = True
+    msg: str
+    while msg := _decode_message(await websocket.recv()):
+        logging.info(msg)
+
+        # HACK: This is meant to handle validation errors. It is a duplicate
+        # of a similar hack implemented in the VS code extension
+        if is_first:
+            is_first = False
+            if msg.startswith(_FAIL_FAST_INDICATOR):
+                return msg
+    return _decode_message(await websocket.recv())
+
+
 class ApiWrapper:
     def __init__(self) -> None:
         self._client = client.client()
@@ -61,21 +86,8 @@ class ApiWrapper:
         async with self._client.establish_websocket_connection(
             path="/api/v1/ws/generate"
         ) as websocket:
-            is_first = True
             await websocket.send(constraints_obj.json())
-
-            while msg := await websocket.recv():
-                logging.info(msg)
-
-                # HACK: This is meant to handle validation errors. It is a duplicate
-                # of a similar hack implemented in the VS code extension
-                if is_first:
-                    is_first = False
-                    _FAIL_FAST_INDICATOR = "{"
-                    if msg.startswith(_FAIL_FAST_INDICATOR):
-                        return generator_result.GenerationResult.parse_raw(msg)
-
-            res = await websocket.recv()
+            res = await _get_return_value_from_websocket(websocket)
             return generator_result.GenerationResult.parse_raw(res)
 
     async def call_execute_quantum_program_task(
@@ -176,7 +188,7 @@ class ApiWrapper:
             path=routes.COMBINATORIAL_OPTIMIZATION_WS_SOLVE_FULL_PATH, ping_interval=20
         ) as websocket:
             await websocket.send(problem.json())
-            res = await websocket.recv()
+            res = await _get_return_value_from_websocket(websocket)
             return execute_result.ExecutionResult.parse_raw(res)
 
     async def call_combinatorial_optimization_operator_task(
