@@ -12,7 +12,7 @@ def enable_jupyter_notebook() -> None:
 #   https://stackoverflow.com/users/2132753/gustavo-bezerra
 def is_notebook() -> bool:
     try:
-        shell = get_ipython().__class__.__name__  # type: ignore
+        shell = get_ipython().__class__.__name__  # type: ignore[name-defined]
         if shell == "ZMQInteractiveShell":
             return True  # Jupyter notebook or qtconsole
         elif shell == "TerminalInteractiveShell":
