@@ -1,9 +1,12 @@
+import logging
+import platform
 import ssl
+import urllib.parse
 from typing import Any, Dict, Optional, Tuple, Union
 
 import httpx
+import pydantic
 import websockets.client
-from loguru import logger
 from semver import VersionInfo
 
 from classiq_interface import __version__ as classiq_interface_version
@@ -23,8 +26,15 @@ _VERSION_UPDATE_SUGGESTION = (
     'Run "pip install -U <PACKAGE>==<REQUIRED VERSION>" to resolve the conflict.'
 )
 
+_logger = logging.getLogger(__name__)
+
+
+class HostVersions(pydantic.BaseModel):
+    classiq_interface: pydantic.StrictStr = pydantic.Field()
+
 
 class Client:
+    _FRONTEND_VARIANT: str = "classiq-sdk"
     _UNKNOWN_VERSION = "0.0.0"
 
     def __init__(self, conf: config.Configuration):
@@ -35,6 +45,12 @@ class Client:
             3600  # Needs to be synced with load-balancer timeout
         )
 
+    @classmethod
+    def _get_user_agent_header(cls) -> Dict[str, str]:
+        return {
+            "User-Agent": f"python/{platform.python_version()} ({platform.platform()}) {cls._FRONTEND_VARIANT}/{CLASSIQ_VERSION}"
+        }
+
     @staticmethod
     def _handle_response(response: httpx.Response) -> Union[Dict, str]:
         if response.is_error:
@@ -46,10 +62,14 @@ class Client:
             if expired:
                 raise ClassiqExpiredTokenError("Expired token.")
 
-            raise ClassiqAPIError(
-                f"Call to API failed with code {response.status_code}: "
-                f"{response.json()['detail']}"
-            )
+            try:
+                message = (
+                    f"Call to API failed with code {response.status_code}: "
+                    f"{response.json()['detail']}"
+                )
+            except BaseException:
+                message = "Call to API failed"
+            raise ClassiqAPIError(message)
 
         return response.json()
 
@@ -57,7 +77,10 @@ class Client:
         return {
             "base_url": self._config.host,
             "timeout": self._HTTP_TIMEOUT_SECONDS,
-            "headers": self._get_authorization_header(),
+            "headers": {
+                **self._get_authorization_header(),
+                **self._get_user_agent_header(),
+            },
         }
 
     async def call_api(
@@ -76,47 +99,52 @@ class Client:
             response = sync_client.request(method=http_method, url=url, json=body)
             return self._handle_response(response)
 
-    def _get_authorization_header(self) -> Dict:
-        access_token = self._token_manager.access_token
+    def _get_authorization_header(self) -> Dict[str, str]:
+        access_token = self._token_manager.get_access_token()
         if access_token is None:
             return dict()
         return {"Authorization": f"Bearer {access_token}"}
 
     def _get_authorization_query_string(self) -> str:
-        access_token = self._token_manager.access_token
+        access_token = self._token_manager.get_access_token()
         if access_token is None:
             return ""
-        return f"?token={access_token}"
+        return urllib.parse.urlencode({"token": access_token})
 
-    def save_tokens(self, access_token: str, refresh_token: Optional[str]) -> None:
-        self._token_manager.save_tokens(access_token, refresh_token)
+    async def update_expired_access_token(self) -> None:
+        await self._token_manager.update_expired_access_token()
 
-    def is_refresh_token_available(self) -> bool:
-        return self._token_manager.is_refresh_token_available()
-
-    def update_expired_access_token(self) -> None:
-        self._token_manager.update_expired_access_token()
-
-    def establish_websocket_connection(self, path: str) -> websockets.client.connect:
+    def establish_websocket_connection(
+        self, path: str, **kwargs
+    ) -> websockets.client.connect:
         _MAX_PAYLOAD_SIZE = 2 ** 23  # = 8MiB ~= 8MB
+        max_size = kwargs.get("max_size", _MAX_PAYLOAD_SIZE)
+        ping_interval = kwargs.get("ping_interval", None)
+        url = urllib.parse.urlsplit(self._config.ws_uri)
+        url = url._replace(path=path, query=self._get_authorization_query_string())
 
         return websockets.client.connect(
-            uri=f"{self._config.ws_uri}{path}{self._get_authorization_query_string()}",
+            uri=url.geturl(),
             ssl=self._ssl_context if self._config.ws_uri.scheme == "wss" else None,
-            max_size=_MAX_PAYLOAD_SIZE,
+            max_size=max_size,
+            ping_interval=ping_interval,
+            extra_headers=self._get_user_agent_header(),
         )
 
-    def get_backend_uri(self):
+    def get_backend_uri(self) -> str:
         return self._config.host
 
     def _get_host_version(self) -> str:
-        versions: Dict[str, str] = self.sync_call_api("get", f"{ROUTE_PREFIX}/versions")
-        return versions["classiq_interface"]
+        host = HostVersions.parse_obj(
+            self.sync_call_api("get", f"{ROUTE_PREFIX}/versions")
+        )
+        return host.classiq_interface
 
     @classmethod
     def _check_matching_versions(
         cls, lhs_version: str, rhs_version: str, normalize: bool = True
     ) -> bool:
+        processed_versions: _PROCESSED_VERSIONS
         if normalize:
             # VersionInfo comparison is compatible with strings but it excludes any build info
             processed_versions = VersionInfo.parse(lhs_version), VersionInfo.parse(
@@ -124,10 +152,9 @@ class Client:
             )
         else:
             processed_versions = lhs_version, rhs_version
-        processed_versions: _PROCESSED_VERSIONS
         if cls._UNKNOWN_VERSION in processed_versions:
             # In case one of those versions is unknown, they are considered equal
-            logger.debug(
+            _logger.debug(
                 "Either {} or {} is an unknown version. Assuming both versions are equal.",
                 lhs_version,
                 rhs_version,
@@ -149,14 +176,24 @@ class Client:
                 f"Classiq API version mismatch: 'classiq' version is {CLASSIQ_VERSION}, "
                 f"'classiq-interface' version is {classiq_interface_version}. {_VERSION_UPDATE_SUGGESTION}"
             )
-        raw_host_version = self._get_host_version()
-        if not self._check_matching_versions(
-            raw_host_version, classiq_interface_version
-        ):
-            raise ClassiqAPIError(
-                f"Classiq API version mismatch: 'classiq-interface' version is "
-                f"{classiq_interface_version}, backend version is {raw_host_version}. {_VERSION_UPDATE_SUGGESTION}"
+
+        try:
+            raw_host_version = self._get_host_version()
+        except httpx.ConnectError:
+            _logger.warning(
+                "Version check failed - host unavailable.",
             )
+        else:
+            if not self._check_matching_versions(
+                raw_host_version, classiq_interface_version
+            ):
+                raise ClassiqAPIError(
+                    f"Classiq API version mismatch: 'classiq-interface' version is "
+                    f"{classiq_interface_version}, backend version is {raw_host_version}. {_VERSION_UPDATE_SUGGESTION}"
+                )
+
+    async def authenticate(self, overwrite: bool) -> None:
+        await self._token_manager.manual_authentication(overwrite=overwrite)
 
 
 DEFAULT_CLIENT: Optional[Client] = None
@@ -167,7 +204,7 @@ def client() -> Client:
     if DEFAULT_CLIENT is None:
         # This call initializes DEFAULT_CLIENT
         configure(config.init())
-
+    assert DEFAULT_CLIENT is not None
     return DEFAULT_CLIENT
 
 
