@@ -31,7 +31,7 @@ class Generator(function_handler.FunctionHandler):
         return wire.Wire
 
     @property
-    def _call_list(self) -> List[function_call.FunctionCall]:
+    def _logic_flow(self) -> List[function_call.FunctionCall]:
         return self._constraints.logic_flow
 
     def generate(self) -> result.GeneratedCircuit:
